;This program checks if the interrupt
;vectors are already installed for
;INT 10h and INT 0xFF, and checks if
;it'already installed.
;;


;Currently only supports standard modes
;2h, 3h, 4h, 5h, 12h, 13h.


;nasm zulia.asm -o zulia.com

org 100h
bits 16



%ixdefine _x86_Portable__PLATFORMBITS_ 1632  ;16 is the best mode for 16-bit 386+
%include "x86.asm"                           ;as it's truly specific to the size
                                             ;we will use in any mode but also
                                             ;lets us use the automatically-sized
                                             ;registers.


 ;Write text mode ZULIA screen
 ;;



 ;If patch is already installed, skip:
 ;;
 ;If WORD at 0000h:10h*4 is == _int10h,
 ;we have already installed it.
 ;
 ;Just display the splash.



 ;Copy interrupt vector.
 ;It's in format:
 ;
 ;Word 0 -- Offset
 ;Word 1 -- Segment
 ;;
  mov esi,10h*4      ;Point to INT 10h vector base address
  xor eax,eax

 ;Set DS:DI to 0:10h*4 for the IDT:
 ;;
  push ds   ;Save DS in ES
  pop es

  push ds        ;Set DS to 0 to access IDT
  push word 0
  pop ds         ;DS:SI base == 0 (real mode IVT)

  mov edi,[esi]  ;Get old INT 10H vector at 0000:10h*4



 ;Previous install check:
 ;;
  cmp di,_int10h            ;See if our segment offset is already installed
  jne .notinstalled

  mov byte[es:instmark],1   ;If so, mark while this instance runs

  jmp .already_installed0   ;If we are here, we had already installed this
  .notinstalled:



 ;Put old INT 10H vector in
 ;INT 0xFF vector:
 ;;
  mov [0xFF<<2],edi     ;Save old SEG:OFF in INT 0xFF

 ;Build the new INT 10H pointer
 ;from this program:
 ;;
  xor edi,edi        ;DWORD
  push cs            ;Use current CS
  pop di             ;Put in DI
  shl edi,16         ;Puth in higher 16 bits of EDI
  mov di,_int10h     ;Put offset to our ISR in DI

  mov dword[esi],edi ;Place new pointer in INT 10H vector


 .already_installed0:
 pop ds    ;Restore original program's DS





 ;See if we had alreadyinstalled above.
 ;If so, don't leave the program as TSR
 ;(it was already present):
 ;;
  cmp byte[es:instmark],1
  je .eend


 ;This is absolutely all we need to call
 ;to install a TSR with an appropriate
 ;amount of reserved memory.
 ;;
  mov ah,0x31      ;Service - terminate and stay resident (freeze execution) for more than 64K
  mov al,0         ;errorlevel
  mov dx,9000/16   ;Size of module in paragraphs from PSP start (before 100h)
  int 21h          ;Call it to leave resident TSR




 ;The only best thing would be to have
 ;a DOS function to reserve specific
 ;physical memory so that we could
 ;reliably install only our ISR in
 ;a free segment with manual copy.



 ;End the whole program:
 ;;
  .eend:
  mov ax,4C00h
  int 21h













instmark db 0







;The order of labels according
;of structure:
;;

align 4
%include "710patch.asm"









